import java.text.*;
import java.util.*;

import aos.jack.jak.util.timer.DilationController;

/**
   The ScheduleRow class holds a "row' of the schedule, which
   corresponds to one aircraft departing at a certain time with a
   "booking ETA", then arraiving to the destination airport control
   area at a later time, with a re-newed ETA.
*/
class ScheduleRow {

    ScheduleRow next;

    String callsign;
    long booking;
    long booking_ETA;
    long arrival;
    long arrival_ETA;

    // A data line has five information items:
    // {0} = the aircraft callsign to use
    // {1} = the booking time
    // {2} = the ETA to use for booking
    // {3} = the arrival time at destination control area
    // {4} = the ETA to use then for ATL renewal
    static MessageFormat syntax = new MessageFormat
	("{0} {1,time,HH:mm} {2,time,HH:mm} {3,time,HH:mm} {4,time,HH:mm}");

    ScheduleRow(TrafficGUI gui,String line)
	throws ParseException
    {
	Object [] field = syntax.parse(line);
	callsign    = (String)field[0];
	booking     = ((Date)field[1]).getTime() + Main.baseTime;
	booking_ETA = ((Date)field[2]).getTime() + Main.baseTime;
	arrival     = ((Date)field[3]).getTime() + Main.baseTime;
	arrival_ETA = ((Date)field[4]).getTime() + Main.baseTime;

	gui.addRow(booking/1000, toString());
    }

    public String toString()
    {
	return callsign+
	    " "+DilationController.timeString(booking)+
	    " "+DilationController.timeString(booking_ETA)+
	    " "+DilationController.timeString(arrival)+
	    " "+DilationController.timeString(arrival_ETA);
    }

}
