package gui;

import java.awt.*;
import java.io.*;
import java.awt.event.*;

import agents.*;
import aos.util.Properties;

public class Machine extends Frame implements AtmInterface {

    AtmClient atm;

    class WindowActions extends WindowAdapter {
	public void windowClosing(WindowEvent e) {
	    System.exit(0);
	}
    }

    TextArea display;
    TextField input;
    int value;

    String message;
    String more;

    void setMessage(String m)
    {
	message = m;
	more = null;
	display.setText(message);
	display.setEditable(false);
	input.requestFocus();
    }

    void error(String m)
    {
	more = m;
	display.setText(message + "\n" + more);
	display.setEditable(false);
	input.requestFocus();
    }

    class Input extends Panel implements ActionListener {

	int base;

	Input(int b)
	{
	    super(new BorderLayout());
	    base = b;
	    input = new TextField("",20);
	    input.addActionListener(this);
	    add(input,"Center");
	    addButton("     clear     ","West");
	    addButton("     OK      ","East");
	}

        void addButton(String x,String y)
        {
            Button b = new Button(x);
            add(b,y);
            b.addActionListener(this);
        }

        public void actionPerformed(ActionEvent e)
        {
	    String cmd =  e.getActionCommand();

	    cmd = cmd.trim();
	    if (cmd.equals("clear")) {
		input.setText("");
		display.setEditable(true);
		input.requestFocus();
		return;
	    } else if (cmd.equals("OK")) {
		cmd = input.getText();
	    }
	    try {
		value = Integer.parseInt(cmd);
		buttonAction(base);
	    }
	    catch (Exception e1) {
		error("Invalid input");
		input.setText("");
	    }
	}
    }

    class ButtonPanel extends Panel implements ActionListener {

	int base;
	String [] buttons;

	ButtonPanel(String [] buttons,int b)
	{
	    super(new GridLayout(buttons.length,1));
	    base = b;
	    this.buttons = buttons;
	    for (int i=0; i<buttons.length; i++)
		addButton(buttons[i]);
	}

	void addButton(String x)
	{
	    Button b = new Button(x);
	    add(b);
	    b.addActionListener(this);
	}

	public void actionPerformed(ActionEvent e)
	{
            for (int i=0; i<buttons.length; i++) {
		if (buttons[i].equals(e.getActionCommand())) {
		    buttonAction(base+i);
		    break;
		}
	    }
	}

    }

    static String [] left_buttons = {
	"Deposit",
	"Statement",
	"Exit"
    };

    static String [] right_buttons = {
	"Withdraw",
	"Transfer",
	"Eject"
    };

    int status1 = 0;
    int status2 = 0;

    boolean card = false;
    int account;
    int pin;
    int amount;

    void buttonAction(int cmd)
    {
	switch (cmd) {
	case 0: // Deposit
	case 1: // Statement
	    error("Service not available");
	    break;
	case 2: // Exit
	    System.exit(0);
	    
	case 3: // Withdraw
	    if (status1 < 2)
		break;
	    status2 = 1;
	    enterAmount("Withdraw from account.");
	    break;
	case 4: // Transfer
	    error("Service not available");
	    break;
	case 5: // Eject
	    card = false;
	    eject();
	    break;
	case 6: // input value;
	    switch (status1) {
	    case 0:
		account = value;
		enterPin();
		break;
	    case 1:
		pin = value;
		card = true;
		atm.insertCard(account,pin);
		selectTransaction();
		break;
	    case 3:
		switch (status2) {
		case 1:
		    amount = value;
		    input.setEditable(false);
		    atm.withdraw(amount);
		    status1 = 4;
		    break;
		}
		status2 = 0;
		break;
	    }
	    break;
	}
    }

    void selectTransaction()
    {
	status1 = 2;
	setMessage("  Welcome to this ATM"+
                   "\nAccount: "+account+
                   "\nPIN: "+pin+
                   "\n\nPlease select service.");
	input.setEditable(false);
	input.setText("");
    }

    void enterAmount(String service)
    {
	status1 = 3;
	setMessage(service+
                   "\nAccount: "+account+
                   "\nPIN: "+pin+
                   "\n\nPlease enter amount and push OK.");
	input.setEditable(true);
	input.setText("");
    }

    void enterPin()
    {
	status1 = 1;
	setMessage("  Welcome to this ATM"+
		   "\nAccount number: "+account+
		   "\n\nPlease enter your PIN code and push OK.");
	input.setEditable(true);
	input.setText("");
    }

    public void eject()
    {
	status1 = 0;
	setMessage("  Welcome to this ATM"+
		   "\n\nPlease enter your account number and push OK.");
	input.setEditable(true);
	input.setText("");
    }


    public Machine()
    {
	super(Properties.getStringProperty("ATM.Title","ATM Demo"));
	addWindowListener(new WindowActions());

	add(new Banner("atm.gif"),"North");
	display = new TextArea("",7,20,TextArea.SCROLLBARS_NONE);
	add(display,"Center");

	add(new Input(left_buttons.length+right_buttons.length),"South");
	//setSize(500,500);
	add(new ButtonPanel(left_buttons,0),"West");
	add(new ButtonPanel(right_buttons,left_buttons.length),"East");
	eject();
	pack();
	show();
	input.requestFocus();
    }

    public static void main(String [] arg)
    {
	new Machine();
    }

    public void register(AtmClient a)
    {
	atm = a;
    }

    public boolean cardInserted()
    {
        return card;
    }

    public void dispense(int b)
    {
	setMessage("Withdraw APPROVED"+
                   "\nAccount: "+account+
			 "\nAmount requested: "+amount+
			 "\nAmount dispensed: "+b);
	input.setEditable(false);
	input.setText("");
   }

    public void message(java.lang.String m)
    {
	  error(m);
    }

    public void receipt(int account, int balance)
    {
	setMessage("Receipt"+
                   "\nAccount: "+account+
			 "\nBalance: "+balance);
	input.setEditable(false);
	input.setText("");
    }
}
