import java.io.*;
import aos.util.PathEntry;		// Find files using the CLASSPATH

/**
   A Schedule is a table of departures. It is represented using "plain
   Java" (rather than a JACK beliefset) because the application does
   not manipulate the schedule in any way, and it is accessed row by
   row, ordered by departure time.
   A Schedule object is also associated with a presentation, which
   merely presents the schedule.
*/
class Schedule {

    TrafficGUI gui;

    ScheduleRow rows;

    void assertLine(String line)
    {
	try {
	    ScheduleRow r = new ScheduleRow(gui,line);
	    ScheduleRow after = null;
	    for (ScheduleRow p = rows; p != null; after = p, p = p.next)
		if (r.booking < p.booking)
		    break;
	    if (after == null) {
		r.next = rows;
		rows = r;
	    } else {
		r.next = after.next;
		after.next = r;
	    }
	}
	catch (Exception e) {
	    System.err.println(e);
	}
    }

    void load(String fileName,TrafficGUI gui)
    {
	this.gui = gui;
	try {
	    // Locate the file using CLASSPATH since we may be
	    // running from a zip file.
	    BufferedReader input =  new BufferedReader(
		new InputStreamReader(PathEntry.open(fileName)));
	    String line;
	    while ((line = input.readLine()) != null) {
		line = line.trim();
		if ((line.length() == 0) || (line.charAt(0) == '#'))
		    continue;
		assertLine(line);
	    }
	    input.close();
	    gui.show();
	}
	catch (IOException e) {
	    // Don't worry ???
	    System.err.println(e);
	}
    }

}
