package jsv.table;

import aos.jack.util.cursor.Action;

/**
 * The VirtualController class provides basic transformation operations
 * (rotate and move). Currently only rotate is implemented.
 */
public class VirtualController { 
    // These variables define the virtual controller state
    double orientation; // orientation of the transform's coordinate system
    double x;		// x location (in the transform's coordinate system)
    double y;		// y location (in the transform's coordinate system)
    
    double da;		// step angle
    int dat;		// rotation time for a step (msecs)
    double ds;		// step distance
    int dst;		// movement time for a step (msecs)

    public VirtualController(
    	double o,double x,double y,double da, int dat, double ds, int dst)
    {
    	orientation = o;
	this.x = x;
	this.y = y;
	this.da = da;
	this.dat = dat;
	this.ds = ds;
	this.dst = dst;
    }

// state variable access

    public double getOrientation() 
    {
    	return orientation;
    }

    public double getX() 
    {
    	return x;
    }

    public double getY() 
    {
    	return y;
    }

    public void setOrientation(double o)
    {
    	orientation = o;
    }

    public void setLocation(double x, double y)
    {
    	this.x = x;
    	this.y = y;
    }

// rotate operation (rotation of coordinate system)

    public Action rotate(double a)
    {
	return new RotateAction(a);
    }

    class RotateAction extends Action {

	double a;	// angle to rotate through (radians)

	RotateAction(double a) {

	    this.a = a;
	}

	protected void action()
	{
	    clockWise( a );
	}
    }

    boolean moreToGo(double from,double to)
    {
	if ( from > to )
	    to += 2 * Math.PI;
	return ( to - from ) > da;
    }

    void clockWise(double a)
    {
	double w = orientation+a;

	while ( w > 2 * Math.PI )
	    w -= 2 * Math.PI;
	while ( w < 0 )
	    w += 2 * Math.PI;

	while ( moreToGo( orientation, w ) ) {
	    orientation +=  da;
	    delay(dat);
	}
	if ( orientation != w ) {
	    orientation = w;
	    delay(dat);
	}
    }

    void delay(int dt)
    {
	try { Thread.sleep( dt ); }
	catch (InterruptedException e) {
	    System.err.println( "TableState: sleep interrupted" );
	}
    }

}
