import java.util.StringTokenizer;
import java.io.*;
import aos.apib.*;		// the JACOB classes
import aos.util.PathEntry;

public class Main
{

    public static void
    main(String [] args)
    {
	// We will only read a maximum of 10 records for this example
	Record[] records = new Record[10];

	records = loadTextRecords("data.txt", 10);

	System.out.println("Data read from data.txt");
	dumpRecords(records);

	XMLOutStream out = null;

	// Create a JACOB XML output stream
	try {
	    out = new XMLOutStream(new FileOutputStream("data.xml"));
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	    System.exit(1);
	}

	// Write the new XML file
	for (int i = 0; records[i] != null && i < records.length; i++) {
	    out.writeObject(records[i]);
	}
	out.close();

	// Throw away the previously read data
	records = new Record[10];

	// We need a dictionary containing the object types
	// defined in classes.api. These are conveniently
	// found in the generated Init__classes.java file.

	TypeDict dict = new TypeDict();
	dict.initialize("Init__classes");

	// We could have created a XMLInStream specifically but the
	// code shown here will work with other JACOB formats such as
	// Binary or Standard as well as XML.
	InStream in = InStream.open("data.xml", dict);
	int i = 0;
	Base obj;

	while ( (obj = in.readObject()) != null ) {
	    records[i] = (Record) obj;		// we know it is a "Record"
	    i++;
	}

	System.out.println("");
	System.out.println("Data read from data.xml");
	dumpRecords(records);
    }

    private static Record[]
    loadTextRecords(String filename, int max)
    {
	Record[]	recs = new Record[max];
	BufferedReader	in = null;
	int		i = 0;
	String		line;

	try {
	    in = new BufferedReader(
		    	new InputStreamReader(PathEntry.open(filename)));

	    while ( i < max && (line = in.readLine()) != null ) {
		StringTokenizer tokenizer = new StringTokenizer(line, "|");
		Record rec = new Record();
		// We choose not to do any error checking for this example.
		rec.name = tokenizer.nextToken();
		rec.address = tokenizer.nextToken();
		rec.age = Integer.parseInt(tokenizer.nextToken());
		rec.phone = tokenizer.nextToken();
		recs[i] = rec;
		i++;
	    }
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	    System.exit(1);
	}

	return recs;
    }

    private static void
    dumpRecords(Record[] records)
    {
	for (int i = 0; records[i] != null && i < records.length; i++) {
	    System.out.println("Name      = " + records[i].name);
	    System.out.println("Address   = " + records[i].address);
	    System.out.println("Phone     = " + records[i].phone);
	    System.out.println("Age       = " + records[i].age);
	    System.out.println("----");
	}
    }

}
